@@ -88,7 +88,8 @@ int main(void)
 		goto out_libusb_close;
 	}
 
-	/* Checking that the configuration has not changed, as suggested in
+	/*
+	 * Checking that the configuration has not changed, as suggested in
 	 * http://libusb.sourceforge.net/api-1.0/caveats.html
 	 */
 	current_configuration = -1;
@@ -100,7 +101,7 @@ int main(void)
 	}
 
 	if (current_configuration != AM7XXX_STORAGE_CONFIGURATION) {
-		fprintf(stderr, "libusb configuration changed (expected: %d, current: %d\n",
+		fprintf(stderr, "libusb configuration changed (expected: %d, current: %d)\n",
 			AM7XXX_STORAGE_CONFIGURATION, current_configuration);
 		ret = -EINVAL;
 		goto out_libusb_release_interface;
